import { createLogger, format, transports } from 'winston';
import { existsSync, mkdirSync } from 'fs';
import 'winston-daily-rotate-file';
import * as dotenv from 'dotenv';
dotenv.config();
let logDirectory = '';
console.log(process.cwd());
if (process.cwd().match(/lib[\\\/](?:.+?)?$/)) {
    logDirectory = '../../logs';
}
else {
    logDirectory = './logs';
}
if (!existsSync(logDirectory)) {
    mkdirSync(logDirectory);
    console.log(`Dir ${logDirectory} doesn't exist - created it!'`);
}
const logger = createLogger({
    level: 'info',
    format: format.combine(format.timestamp({
        format: 'YYYY-MM-DD HH:mm:ss'
    }), format.errors({ stack: true }), format.printf(infoObj => `${infoObj.timestamp} [${infoObj.level}] ${infoObj.message}`)),
    transports: []
});
if (process.env.NODE_ENV === 'PRODUCTION') {
    logger.add(new transports.DailyRotateFile({
        filename: `${logDirectory}/Log-%DATE%.log`,
        datePattern: 'YYYY-MM-DD',
        zippedArchive: true,
        maxSize: '20m',
        maxFiles: '30d',
        handleExceptions: true
    }));
    logger.add(new transports.Console({
        format: format.combine(format.colorize(), format.printf(infoObj => `${infoObj.timestamp} [${infoObj.level}] ${infoObj.message}`)),
        handleExceptions: true
    }));
}
else {
    logger.add(new transports.Console({
        format: format.combine(format.colorize(), format.printf(infoObj => `${infoObj.timestamp} [${infoObj.level}] ${infoObj.message}`)),
        handleExceptions: true
    }));
}
export function warn(msg) {
    if (process.env.NODE_ENV !== 'TESTING') {
        logger.log({ message: msg, level: 'warn' });
    }
}
export function debug(msg) {
    if (process.env.NODE_ENV !== 'TESTING') {
        logger.log({ message: msg, level: 'debug' });
    }
}
export function info(msg) {
    if (process.env.NODE_ENV !== 'TESTING') {
        logger.log({ message: msg, level: 'info' });
    }
}
