var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as logger from 'logger';
import exceptionalError from 'exceptionalhandler';
import * as slackConstants from './slacker.constants.js';
import util from 'util';
export function performApiReq(reqUrl, method, body) {
    return __awaiter(this, void 0, void 0, function* () {
        let fetchOpts = {
            method: method,
            headers: {}
        };
        if (body && method === slackConstants.slackApiMethodPost) {
            fetchOpts.body = JSON.stringify(body);
            fetchOpts.headers['Content-Type'] = 'application/json';
        }
        return yield processResponse(yield fetch(reqUrl, fetchOpts));
    });
}
function processResponse(apiResp) {
    switch (apiResp.status) {
        case 200:
            return true;
            break;
        case 400:
            handleError(apiResp);
            break;
        case 404:
            handleError(apiResp);
            break;
        default:
            throw new exceptionalError(`Can't handle ${apiResp.status} right now!`);
    }
}
function handleError(apiResp) {
    let errString = `Request returned ${apiResp.status}`;
    if (apiResp.statusText) {
        errString = `${errString} ${apiResp.statusText}`;
    }
    logger.info(util.inspect(apiResp.body));
    throw new exceptionalError(errString);
}
