var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as logger from 'logger';
export { logger };
import exceptionalError from 'exceptionalhandler';
export { exceptionalError };
import * as slackConstants from './slacker.constants.js';
import * as slackApi from './slacker.api.js';
import * as slackUtils from './slacker.utilities.js';
export class slackNotification {
    constructor(name, priority, webhook) {
        this.serviceName = "";
        this.msgArray = [];
        this.webhookUrl = "";
        this.medIcon = "🤔";
        this.highIcon = "🚨";
        this.lowIcon = "ℹ️";
        this.validateNew(name, priority, webhook);
        this.serviceName = name;
        this.webhookUrl = `${slackConstants.slackApiBaseUrl}${webhook}`;
        let newHeader = this.truncateText(name, slackConstants.maxHeaderLength - 5);
        switch (priority) {
            case slackConstants.priorityLow:
                newHeader = `${newHeader} ${this.lowIcon}`;
                break;
            case slackConstants.priorityMedium:
                newHeader = `${newHeader} ${this.medIcon}`;
                break;
            case slackConstants.priorityHigh:
                newHeader = `${newHeader} ${this.highIcon}`;
                break;
        }
        this.addheader(newHeader);
    }
    addheader(headerText) {
        if (slackUtils.valIsEmpty(headerText)) {
            throw new exceptionalError("Text arg required");
        }
        headerText = this.truncateText(headerText, slackConstants.maxHeaderLength);
        let header = {
            type: slackConstants.msgBlockTypeHeader,
            text: {
                type: slackConstants.msgBlockPlainText,
                text: headerText,
                emoji: true
            }
        };
        this.msgArray.push(header);
    }
    addMessage(text, textFormatting = slackConstants.msgBlockPlainText) {
        if (slackUtils.valIsEmpty(text)) {
            throw new exceptionalError("Text arg required");
        }
        text = this.truncateText(text, slackConstants.maxTextFieldLength);
        let newSection = {
            type: slackConstants.msgBlockTypeSection,
            text: {
                type: textFormatting,
                text: text
            }
        };
        if (textFormatting === slackConstants.msgBlockPlainText) {
            newSection.text.emoji = true;
        }
        this.msgArray.push(newSection);
    }
    addTextColumns(textFields, textFormatting = slackConstants.msgBlockPlainText) {
        let fieldsToAdd = [];
        let newFieldsElement = {
            type: slackConstants.msgBlockTypeSection,
            fields: []
        };
        if (!Array.isArray(textFields) || textFields.length === 0) {
            throw new exceptionalError("Must include an array of strings");
        }
        for (let textToAdd of textFields) {
            if (newFieldsElement.fields.length >= slackConstants.maxColumnCount) {
                fieldsToAdd.push(newFieldsElement);
                newFieldsElement = {
                    type: slackConstants.msgBlockTypeSection,
                    fields: []
                };
            }
            if (!textToAdd || textToAdd === "") {
                throw new exceptionalError("All textFields must be non-empty strings");
            }
            textToAdd = this.truncateText(textToAdd, slackConstants.maxColumnTextLength);
            let newTextField = {
                type: textFormatting,
                text: textToAdd
            };
            if (textFormatting === slackConstants.msgBlockPlainText) {
                newTextField.emoji = true;
            }
            newFieldsElement.fields.push(newTextField);
        }
        fieldsToAdd.push(newFieldsElement);
        for (const newFieldEmenet of fieldsToAdd) {
            this.msgArray.push(newFieldEmenet);
        }
    }
    addDivider() {
        this.msgArray.push({ type: slackConstants.msgBlockTypeDivider });
    }
    addImage(size, imageUrl, imageAltText, imageText, textFormatting = slackConstants.msgBlockPlainText) {
        let newImage;
        if (slackUtils.valIsEmpty(size)) {
            throw new exceptionalError(`Size must be big or small`);
        }
        if (slackUtils.valIsEmpty(imageUrl)) {
            throw new exceptionalError(`Image URL required`);
        }
        if (imageUrl.length > slackConstants.maxUrlLength) {
            throw new exceptionalError(`URL too long - max length ${slackConstants.maxUrlLength}`);
        }
        if (!imageAltText) {
            imageAltText = "";
        }
        imageAltText = this.truncateText(imageAltText, slackConstants.maxImageTextLength);
        imageText = this.truncateText(imageText, slackConstants.maxImageTextLength);
        switch (size) {
            case "big":
                newImage = {
                    type: slackConstants.msgBlockTypeImage,
                    image_url: imageUrl,
                    alt_text: imageAltText
                };
                if (!slackUtils.valIsEmpty(imageText)) {
                    newImage.title = {
                        type: textFormatting,
                        text: imageText
                    };
                    if (textFormatting === slackConstants.msgBlockPlainText) {
                        newImage.title.emoji = true;
                    }
                }
                break;
            case "small":
                newImage = {
                    type: slackConstants.msgBlockTypeSection,
                    accessory: {
                        type: slackConstants.msgBlockTypeImage,
                        image_url: imageUrl,
                        alt_text: imageAltText
                    }
                };
                if (!slackUtils.valIsEmpty(imageText)) {
                    newImage.text = {
                        type: textFormatting,
                        text: imageText,
                    };
                    if (textFormatting === slackConstants.msgBlockPlainText) {
                        newImage.text.emoji = true;
                    }
                }
                break;
            default:
                throw new exceptionalError(`Invalid size type ${size}`);
        }
        this.msgArray.push(newImage);
    }
    addButton(maintext, buttonText, url, textFormatting = slackConstants.msgBlockPlainText) {
        if (slackUtils.valIsEmpty(url) || url.length > (0.8 * slackConstants.maxImageTextLength)) {
            logger.info(`URL: ${url}`);
            throw new exceptionalError(`Invalid value provided for URL`);
        }
        maintext = this.truncateText(maintext, slackConstants.maxImageTextLength);
        buttonText = this.truncateText(buttonText, slackConstants.maxImageTextLength);
        if (slackUtils.valIsEmpty(buttonText)) {
            buttonText = "Click Me";
        }
        let newButton = {
            "type": "section",
            "text": {
                "type": textFormatting,
                "text": maintext
            },
            "accessory": {
                "type": "button",
                "text": {
                    "type": slackConstants.msgBlockPlainText,
                    "text": buttonText
                },
                "url": url,
            }
        };
        if (textFormatting === slackConstants.msgBlockPlainText) {
            newButton.text.emoji = true;
        }
        this.msgArray.push(newButton);
    }
    validateNew(name, priority, webhook) {
        if (slackUtils.valIsEmpty(name)) {
            throw new exceptionalError("Service name is a mandatory field");
        }
        if (slackUtils.valIsEmpty(webhook)) {
            throw new exceptionalError("Webhook URL required");
        }
        if (webhook.match(/^\//)) {
            throw new exceptionalError("Webhook should not start with a slash");
        }
        if (!priority || !slackConstants.messagePriorities.includes(priority)) {
            throw new exceptionalError(`Invalid priority "${priority}"`);
        }
    }
    sendNotification() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.msgArray.length === 0) {
                throw new exceptionalError("Can't send an empty message!");
            }
            let msgsToSend = [];
            let newMsgJSON = {
                blocks: []
            };
            let cumulativeCharacters = 0;
            for (const msg of this.msgArray) {
                if (newMsgJSON.blocks.length > slackConstants.maxComponentsPerMessage) {
                    msgsToSend.push(newMsgJSON);
                    newMsgJSON = {
                        blocks: []
                    };
                }
                newMsgJSON.blocks.push(msg);
            }
            msgsToSend.push(newMsgJSON);
            for (const sendMe of msgsToSend) {
                yield slackApi.performApiReq(this.webhookUrl, slackConstants.slackApiMethodPost, sendMe);
            }
            return true;
        });
    }
    truncateText(text, length) {
        const truncatePadding = 5;
        if (text) {
            if (text.length > length) {
                text = `${text.substring(0, length - truncatePadding)}...`;
            }
        }
        return text;
    }
}
